from sql_orm import DATABASE_TYPES, BaseField
from sql_orm.postgresql import sql


class Field(BaseField):

    database_type = DATABASE_TYPES["PostgreSQL"]
    field_type = None
    __value = None

    def __init__(self, verbose_name=None, null=False, unique=False, primary_key=False, default=None, extra_sql=()):
        self.verbose_name = verbose_name
        self.primary_key = primary_key
        if primary_key:
            self.properties = "PRIMARY KEY"
        else:
            if default is not None:
                self.properties = "DEFAULT {}".format("TRUE" if default else "FALSE")
            else:
                self.properties = "NULL" if null else "NOT NULL"
                if unique:
                    self.properties = "UNIQUE " + self.properties
        if extra_sql:
            self.properties += " " + " ".join(extra_sql)
        self.base_create_query = sql.add_table_column()

    @staticmethod
    def convert(value):
        return value

    def __set__(self, instance, value):
        self.__value = self.convert(value) if value is not None else None

    def __get__(self, instance, owner):
        return self.__value

    def create(self, schema, table_name, column_name):
        query = self.base_create_query.format(
            schema=schema,
            table_name=table_name,
            name=column_name,
            field_type=self.field_type,
            properties=self.properties
        )
        return query


class BooleanField(Field):

    field_type = "BOOLEAN"

    @staticmethod
    def convert(value):
        return bool(value)


class IntegerField(Field):

    field_type = "INTEGER"

    @staticmethod
    def convert(value):
        return int(value)


class FloatField(Field):

    field_type = "NUMERIC(15,6)"

    @staticmethod
    def convert(value):
        return float(value)


class DefaultPrimaryKeyField(IntegerField):

    field_type = "SERIAL"

    def __init__(self, verbose_name=None):
        super().__init__(verbose_name=verbose_name, primary_key=True)

    @staticmethod
    def convert(value):
        return int(value)


class DateField(Field):

    field_type = "DATE"


class CharField(Field):

    field_type = "VARCHAR"

    def __init__(self, max_length, verbose_name=None, null=False, unique=False):
        super().__init__(verbose_name=verbose_name, null=null, unique=unique)
        self.properties = "({}) {}".format(max_length, self.properties)

    @staticmethod
    def convert(value):
        return str(value)


class ForeignKeyField(Field):

    def __init__(self, table_name, verbose_name=None, null=False, unique=False):
        pk = table_name.get_pk_name()
        self.table_name = table_name
        self.field = table_name.__dict__[pk]
        extra_sql = "REFERENCES {schema}.{table_name}({pk})".format(
            schema=table_name.get_schema(),
            table_name=table_name.get_table_name(),
            pk=pk
        )
        self.field_type = "INTEGER" if self.field.field_type == "SERIAL" else self.field.field_type
        super().__init__(verbose_name=verbose_name, null=null, unique=unique, extra_sql=(extra_sql, ))

    def __set__(self, instance, value):
        self.__value = value

    def __get__(self, instance, owner):
        if self.__value is not None:
            value = self.table_name.get_value_or_object_pk(self.__value)
            value = self.table_name.objects.get(pk=self.field.convert(value))
            return value
        return None



import psycopg2
import configparser

CONFIG = configparser.ConfigParser()
CONFIG.read("config.ini")


class PostgreSQL:

    def __init__(self):
        credentials = {
            "host": CONFIG["POSTGRESQL"]["DB_HOST"],
            "port": int(CONFIG["POSTGRESQL"]["DB_PORT"]),
            "database": CONFIG["POSTGRESQL"]["DB_NAME"],
            "user": CONFIG["POSTGRESQL"]["DB_USER"],
            "password": CONFIG["POSTGRESQL"]["DB_PASSWORD"]
        }
        self.debug = CONFIG["POSTGRESQL"].get("DEBUG") == "True"
        try:
            self._conn = psycopg2.connect(**credentials)
            self._cursor = self._conn.cursor()
            if self.debug:
                print("\nConnected to PostgreSQL\n")
        except psycopg2.Error as error:
            raise ValueError("Unable to connect to PostgreSQL database\n{error}".format(error=error))

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def __del__(self):
        self.close()

    def close(self):
        self.connection.close()
        if self.debug:
            print("\nClosed PostgreSQL connection.\n")

    @property
    def connection(self):
        return self._conn

    @property
    def cursor(self):
        return self._cursor

    def commit(self):
        self.connection.commit()

    def query(self, sql, params=None):
        if self.debug:
            print(self.mogrify(sql, params))
        self.cursor.execute(sql, params or ())

    def mogrify(self, sql, params=None):
        return self.cursor.mogrify(sql, params or ())

    def fetchall(self):
        return self.cursor.fetchall()

    def fetchone(self):
        return self.cursor.fetchone()

    def insert(self, sql, params=None):
        if self.debug:
            print(self.mogrify(sql, params))
        self.cursor.execute(sql, params or ())
        self.commit()

    def insert_many(self, sql, params=None):
        params_str = ','.join((self.mogrify("%s", (x, ))).decode('utf-8') for x in params)
        if self.debug:
            print(self.mogrify(sql.format(params_str)))
        self.cursor.execute(sql.format(params_str))
        self.commit()

    def fetch_query_results(self, sql, params=None):
        if self.debug:
            print(self.mogrify(sql, params))
        self.cursor.execute(sql, params or ())
        while True:
            try:
                results = self.cursor.fetchmany(100)
                if not results:
                    break
                for result in results:
                    yield result
            except psycopg2.ProgrammingError:
                break



from collections import OrderedDict


class InvalidQueryException(Exception):
    pass


def create_schema():
    return "CREATE SCHEMA IF NOT EXISTS {name};"


def create_table():
    return "CREATE TABLE IF NOT EXISTS {schema}.{name} ({add_primary_key});"


def add_table_column():
    return "ALTER TABLE {schema}.{table_name} ADD COLUMN IF NOT EXISTS {name} {field_type} {properties};"


def insert_table_row():
    return "INSERT INTO {schema}.{table_name} ({column_names}) VALUES ({column_values}) RETURNING id;"


def update_table_row():
    return "UPDATE {schema}.{table_name} SET {set_key_value} WHERE {condition};"


def add_unique_together():
    return "ALTER TABLE {schema}.{table_name} ADD CONSTRAINT {constraint_name} UNIQUE ({columns});"


LOGICAL_SEPARATOR = "__"


class Query:

    def __init__(self, schema, table_class, table_columns, pk, order_dict=None, filter_dict=None, or_filter_dict=None, exclude_dict=None, limit=None, offset=None, delete=False):
        self.__schema = schema
        self.__table_class = table_class
        self.__table_name = table_class.get_table_name()
        self.__table_columns = table_columns
        self.__pk = pk
        self.__order_dict = order_dict if order_dict else {"id": "ASC"}
        self.__filter_dict = filter_dict
        self.__or_filter_dict = or_filter_dict
        self.__exclude_dict = exclude_dict
        self.__limit = limit
        self.__offset = offset
        self.__delete = delete
        self.__operators = {
            "gt": ">",
            "gte": ">=",
            "lt": "<",
            "lte": "<=",
            "exact": "=",
            "iexact": "=",
            "contains": "LIKE",
            "icontains": "ILIKE",
            "startswith": "LIKE",
            "istartswith": "ILIKE",
            "endswith": "LIKE",
            "iendswith": "ILIKE",
            "isnull": "IS",
        }
        self.__params = []
        self.__base_query = "SELECT {};" if not delete else "DELETE {};"
        self.__from_query = ""
        self.__where_query = ""
        self.__order_by_query = ""
        self.__limit_offset_query = ""
        self.__tables_involved = OrderedDict()

    def __logical_conditions(self, schema, table_name, key, value, condition):
        if key == "pk":
            key = self.__pk
        key = "{}.{}.{}".format(schema, table_name, key)
        if condition == "=":
            self.__params.append(value)
            return "{key}=%s".format(key=key)
        if condition == "isnull":
            base = "{key} {operation}".format(
                key=key,
                operation=self.__operators[condition]
            )
            return "{} NULL".format(base) if value else "{} NOT NULL".format(base)
        if condition == "iexact":
            self.__params.append(value)
            return "LOWER({key}){operation}LOWER(%s)".format(
                key=key,
                operation=self.__operators[condition],
            )
        if condition in ("contains", "icontains", "startswith", "istartswith", "endswith", "iendswith"):
            if condition == "contains" or condition == "icontains":
                self.__params.append("%{}%".format(value))
            elif condition == "startswith" or condition == "istartswith":
                self.__params.append("{}%".format(value))
            elif condition == "endswith" or condition == "iendswith":
                self.__params.append("%{}".format(value))
            return "{key} {operation} %s".format(
                key=key,
                operation=self.__operators[condition],
                value=value,
            )
        self.__params.append(value)
        return "{key}{operation}%s".format(
            key=key,
            operation=self.__operators[condition],
        )

    def __set_tables_involved(self, table_class, key=None):
        table = "{}.{}".format(table_class.get_schema(), table_class.get_table_name())
        if table in self.__tables_involved:
            if key and not self.__tables_involved[table]["key"]:
                self.__tables_involved[table].update({"key": key})
        else:
            self.__tables_involved[table] = {
                "schema": table_class.get_schema(),
                "table": table_class.get_table_name(),
                "columns": table_class.get_column_names(),
                "pk": table_class.get_pk_name(),
                "key": key,
            }

    def __switch_to_join_query(self):
        tables = list(self.__tables_involved.items())
        if self.__delete:
            from_query = "FROM {} USING ".format(tables[0][0])
            self.__from_query = from_query + ", ".join([i[0] for i in tables[1:]])
        else:
            column_query = ", ".join(",".join(["{}.{}.{}".format(i["schema"], i["table"], j) for j in i["columns"]]) for _, i in tables)
            join_query = "{} FROM {}.{}".format(column_query, tables[0][1]["schema"], tables[0][1]["table"])
            for i in range(1, len(tables)):
                join_query += (
                    " LEFT JOIN " +
                    "{}.{}".format(tables[i][1]["schema"], tables[i][1]["table"]) +
                    " ON " +
                    "{}={}".format("{}.{}.{}".format(tables[i-1][1]["schema"], tables[i-1][1]["table"], tables[i-1][1]["key"]),
                                   "{}.{}.{}".format(tables[i][1]["schema"], tables[i][1]["table"], tables[i][1]["pk"]))
                )
            self.__from_query = join_query

    def __change_to_sql_conditions(self, key, value):
        key_splits = key.split(LOGICAL_SEPARATOR)
        if len(key_splits) == 1:
            self.__set_tables_involved(table_class=self.__table_class)
            return self.__logical_conditions(
                schema=self.__schema,
                table_name=self.__table_name,
                key=key_splits[0],
                value=value,
                condition="="
            )
        else:
            if key_splits[-1] in self.__operators.keys():
                condition = key_splits[-1]
                key = key_splits[-2]
                fk_fields = key_splits[:-2]
                if not fk_fields:
                    self.__set_tables_involved(table_class=self.__table_class)
                    return self.__logical_conditions(
                        schema=self.__schema,
                        table_name=self.__table_name,
                        key=key,
                        value=value,
                        condition=condition
                    )
            else:
                key = key_splits[-1]
                condition = "="
                fk_fields = key_splits[:-1]

            fk_table_class = self.__table_class
            self.__set_tables_involved(table_class=self.__table_class, key=fk_fields[0])
            for i in range(len(fk_fields)):
                fk = fk_fields[i]
                if not hasattr(fk_table_class.__dict__[fk], "table_name"):
                    raise InvalidQueryException("Not a ForeignKey: {}".format(fk))
                fk_table_class = getattr(fk_table_class.__dict__[fk], "table_name")
                self.__set_tables_involved(table_class=fk_table_class, key=None if i == len(fk_fields) -1 else fk_fields[i+1])

            return self.__logical_conditions(
                schema=fk_table_class.get_schema(),
                table_name=fk_table_class.get_table_name(),
                key=key,
                value=value,
                condition=condition
            )

    def __create_where_query(self):
        filter_query = ""
        if self.__or_filter_dict:
            filter_query = "( {} )".format(" OR ".join([
                self.__change_to_sql_conditions(k, v)
                for k, v in self.__or_filter_dict.items()]))
        if self.__filter_dict:
            filter_query += " AND ".join([
                self.__change_to_sql_conditions(k, v)
                for k, v in self.__filter_dict.items()])
        if self.__exclude_dict:
            if self.__filter_dict:
                filter_query += " AND "
            filter_query += "NOT "
            filter_query += " AND NOT ".join([
                self.__change_to_sql_conditions(k, v)
                for k, v in self.__exclude_dict.items()])
        if filter_query:
            filter_query = " WHERE {}".format(filter_query)
        self.__where_query = filter_query

    def __create_from_query(self):
        if len(self.__tables_involved.keys()) <= 1:
            if self.__delete:
                self.__from_query = "FROM {}.{}".format(self.__schema, self.__table_name)
            else:
                self.__from_query = "{} FROM {}".format(
                    ", ".join(["{}.{}.{}".format(self.__schema, self.__table_name, i) for i in self.__table_columns]),
                    "{}.{}".format(self.__schema, self.__table_name)
                )
        else:
            self.__switch_to_join_query()

    def __create_order_by_query(self):
        order_query = ""
        if not self.__delete:
            if self.__order_dict:
                order_query = ", ".join(
                    ["{}.{}.{} {}".format(self.__schema, self.__table_name, k, v) for k, v in self.__order_dict.items()])
            if order_query:
                order_query = " ORDER BY {}".format(order_query)
        self.__order_by_query = order_query

    def __create_limit_offset_query(self):
        query = ""
        if self.__limit is not None:
            query += " LIMIT {}".format(self.__limit)
        if self.__offset:
            query += " OFFSET {}".format(self.__offset)
        self.__limit_offset_query = query

    def query(self):
        self.__create_order_by_query()
        self.__create_limit_offset_query()

        self.__create_where_query()
        self.__create_from_query()

        query = (
            self.__from_query +
            self.__where_query +
            self.__order_by_query +
            self.__limit_offset_query
        )
        return self.__base_query.format(query), self.__params



from sql_orm import postgresql
from sql_orm.postgresql import sql
from sql_orm.postgresql import datatypes


class QueryException(Exception):
    pass


class ObjectDoesNotExist(Exception):
    pass


class MultipleObjectsFound(Exception):
    pass


class RowSet:

    def __init__(self, table_class):
        self.pgsql = postgresql.PostgreSQL()
        self.__table_class = table_class
        self.__table_columns = table_class.get_column_names()
        self.__filter_exclude_inputs = {
            "filter": {},
            "or_filter": {},
            "exclude": {},
            "order_by": {}
        }
        self.__limit = None
        self.__offset = None
        self.__delete = False

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def __del__(self):
        self.close()

    def close(self):
        self.pgsql.close()

    def __sql_read(self, query, params=()):
        for i in self.pgsql.fetch_query_results(query, params=params):
            yield i

    def __sql_delete(self, query, params=()):
        self.pgsql.query(query, params=params)
        self.pgsql.commit()

    def __update_query_inputs(self, data):
        if data:
            for k, v in data.items():
                self.__filter_exclude_inputs[k].update(v)

    def __create_query(self):
        query = sql.Query(
            schema=self.__table_class.get_schema(),
            table_class=self.__table_class,
            table_columns=self.__table_columns,
            pk=self.__table_class.get_pk_name(),
            order_dict=self.__filter_exclude_inputs["order_by"],
            filter_dict=self.__filter_exclude_inputs["filter"],
            or_filter_dict=self.__filter_exclude_inputs["or_filter"],
            exclude_dict=self.__filter_exclude_inputs["exclude"],
            limit=self.__limit,
            offset=self.__offset,
            delete=self.__delete
        )
        sql_query, params = query.query()
        return {"query": sql_query, "params": params}

    def __getitem__(self, index):
        if isinstance(index, slice):
            if index.step:
                print("WARNING: step provided will be ignored. Not yet supported in PostgreSQL")
            start = int(index.start) if index.start else 0
            if start < 0:
                raise ValueError("Start index cannot be negative.")
            self.__offset = start

            stop = int(index.stop) if index.stop else None
            if stop:
                if stop < start:
                    raise ValueError("Stop index cannot be negative and less than Start index.")
                self.__limit = stop - start
            return self.__iter__()
        if isinstance(index, int):
            if index < 0:
                raise ValueError("Index cannot be negative.")
            self.__limit = 1
            self.__offset = index
            return [i for i in self.__iter__()]
        raise ValueError("Invalid index.")

    def __iter__(self):
        for i in self.__sql_read(**self.__create_query()):
            properties = {self.__table_columns[col]: i[col] for col in range(len(self.__table_columns))}
            obj = self.__table_class(**properties)
            yield obj

    def __next__(self):
        return next(self.__iter__())

    def __validate_kwargs(self, kwargs):
        for key in kwargs.keys():
            column_name = key.rsplit(sql.LOGICAL_SEPARATOR, 1)[0]
            if column_name not in self.__table_columns:
                raise ValueError("The column could not be found. {}".format(column_name))

    def create(self, **kwargs):
        obj = self.__table_class(**kwargs)
        obj.save(commit=True)
        return obj

    def order_by(self, params):
        data = {}
        if type(params) == str:
            params = (params, )
        for i in params:
            order = "DESC" if i[0] == "-" else "ASC"
            column_name = i[1:] if i[0] == "-" else i
            if column_name not in self.__table_columns:
                raise QueryException("Column not found: {}".format(column_name))
            data[column_name] = order
        self.__update_query_inputs({"order_by": data})
        return self

    def all(self):
        self.__update_query_inputs({})
        return self

    def filter(self, **kwargs):
        self.__update_query_inputs({"filter": kwargs})
        return self

    def or_filter(self, **kwargs):
        self.__update_query_inputs({"or_filter": kwargs})
        return self

    def exclude(self, **kwargs):
        self.__update_query_inputs({"exclude": kwargs})
        return self

    def get(self, **kwargs):
        self.__filter_exclude_inputs["filter"] = kwargs
        objects_found = [i for i in self.__iter__()]
        if not objects_found:
            raise ObjectDoesNotExist("Object does not exist.")
        if len(objects_found) > 1:
            raise MultipleObjectsFound("Multiple objects found.")
        return objects_found[0]

    def get_or_create(self, **kwargs):
        created = False
        try:
            obj = self.get(**kwargs)
        except ObjectDoesNotExist:
            obj = self.create(**kwargs)
            created = True
        except MultipleObjectsFound as e:
            raise e
        return obj, created

    def get_or_none(self, **kwargs):
        try:
            obj = self.get(**kwargs)
            return obj
        except Exception as e:
            pass
        return None

    def delete(self):
        self.__delete = True
        self.__sql_delete(**self.__create_query())


class Objects:

    def __get__(self, instance, owner):
        self.row_set = RowSet(
            table_class=owner,
        )
        return self.row_set



from sql_orm import postgresql, DATABASE_TYPES, SQLException, Table
from sql_orm.postgresql import sql
from sql_orm.postgresql.objects import Objects


class PostgreSQLTable(Table):

    _schema = "public"
    objects = Objects()

    def __init__(self, **kwargs):
        super().__init__(database_type=DATABASE_TYPES["PostgreSQL"], **kwargs)

    @classmethod
    def get_schema(cls):
        return cls._schema

    @classmethod
    def __create_schema(cls):
        return sql.create_schema().format(name=cls.get_schema())

    @classmethod
    def __create_table(cls):
        add_primary_key = ""
        for k, v in cls._get_column_fields().items():
            if v.primary_key:
                if not add_primary_key:
                    add_primary_key = "{name} {field_type} {properties}".format(
                        name=k,
                        field_type=v.field_type,
                        properties=v.properties
                    )
                else:
                    raise SQLException("Multiple primary keys found.")
        if not add_primary_key:
            raise SQLException("No primary key found.")
        return sql.create_table().format(
            schema=cls.get_schema(),
            name=cls.get_table_name(),
            add_primary_key=add_primary_key
        )

    @classmethod
    def __create_columns(cls):
        schema = cls.get_schema()
        table_name = cls.get_table_name()
        for k, v in cls._get_column_fields().items():
            if v.primary_key:
                continue
            if v.database_type != DATABASE_TYPES["PostgreSQL"]:
                raise SQLException("Mismatch database fields and table types. Field: ")
            yield v.create(schema=schema, table_name=table_name, column_name=k)

    @classmethod
    def __create_meta_properties(cls):
        meta_queries = []
        meta_field = cls._get_meta_field()
        column_names = cls.get_column_names()
        if meta_field:
            unique_together = meta_field.__dict__.get("unique_together", ())
            for i in unique_together:
                for j in i:
                    if j not in column_names:
                        raise SQLException("Column {} does not exist in the model {}.".format(j, cls.get_table_name()))
                meta_queries.append(sql.add_unique_together().format(
                    schema=cls.get_schema(),
                    table_name=cls.get_table_name(),
                    constraint_name="{}_uniq".format("_".join(i)),
                    columns=",".join(i)
                ))
        return meta_queries

    @classmethod
    def get_value_or_object_pk(cls, value):
        return getattr(value, "pk") if hasattr(value, "pk") else value

    def __get_field_value(self, field_name):
        value = getattr(self, field_name)
        return self.__class__.get_value_or_object_pk(value)

    @classmethod
    def migrate(cls, dry_run=False):
        queries = (
            ["BEGIN;", cls.__create_schema(), cls.__create_table()] +
            [i for i in cls.__create_columns()] +
            cls.__create_meta_properties() +
            ["COMMIT;"]
        )
        with postgresql.PostgreSQL() as pgsql:
            for query in queries:
                if dry_run:
                    print(pgsql.mogrify(query))
                else:
                    pgsql.query(query)
                    pgsql.commit()

    @property
    def pk(self):
        for k, v in self.__class__._get_column_fields().items():
            if v.primary_key:
                return getattr(self, k)

    @classmethod
    def get_pk_name(cls):
        for k, v in cls._get_column_fields().items():
            if v.primary_key:
                return k

    def _sql_save(self, commit=True):
        if getattr(self, "pk"):
            pk_name = self.__class__.get_pk_name()
            column_names = [i for i in self.__class__.get_column_names() if i != pk_name]
            params = [self.__get_field_value(i) for i in column_names] + [self.pk]
            query = sql.update_table_row().format(
                schema=self.__class__.get_schema(),
                table_name=self.__class__.get_table_name(),
                set_key_value=", ".join(["{}=%s".format(i) for i in column_names]),
                condition="{}=%s".format(pk_name)
            )
            with postgresql.PostgreSQL() as pgsql:
                pgsql.query(query, params=params)
                pgsql.commit()
        else:
            column_names = [i for i in self.__class__.get_column_names() if i != "id"]
            params = [self.__get_field_value(i) for i in column_names]
            query = sql.insert_table_row().format(
                schema=self.__class__.get_schema(),
                table_name=self.__class__.get_table_name(),
                column_names=", ".join(column_names),
                column_values=", ".join(["%s"] * len(column_names))
            )
            obj_id = None
            if commit:
                with postgresql.PostgreSQL() as pgsql:
                    pgsql.query(query, params=params)
                    obj_id = pgsql.fetchone()[0]
                    pgsql.commit()
            setattr(self, "id", obj_id)

    def save(self, commit=True):
        self._sql_save(commit=commit)

    def delete(self):
        if getattr(self, "pk"):
            self.__class__.objects.filter(pk=self.pk).delete()
        else:
            raise SQLException("Missing primary key for the given object.")

    def as_dict(self):
        return {k: getattr(self, k) for k in self.__class__.get_column_names()}


import inspect


DATABASE_TYPES = {
    "PostgreSQL": "postgresql",
}


class SQLException(Exception):
    pass


NON_COLUMN_FIELDS = ("__module__", "__doc__", "Meta", "_schema")


class BaseField:
    pass


class Table:

    database_type = None

    def __init__(self, database_type, **kwargs):
        if database_type not in DATABASE_TYPES.values():
            raise SQLException("Wrong database_type. Valid options: {}".format(", ".join(DATABASE_TYPES.values())))
        self.database_type = database_type
        for i in self.__class__.get_column_names():
            setattr(self, i, kwargs.get(i))

    @classmethod
    def _get_column_fields(cls):
        valid_column_names = cls.get_column_names()
        return {k: v for k, v in cls.__dict__.items() if k in valid_column_names}

    @classmethod
    def _get_meta_field(cls):
        return cls.__dict__.get("Meta")

    @classmethod
    def get_column_names(cls):
        names = []
        for k, v in cls.__dict__.items():
            class_inspects = inspect.getmro(v.__class__)
            if len(class_inspects) > 1 and class_inspects[-2] == BaseField:
                names.append(k)
        return sorted(names)

    @classmethod
    def get_table_name(cls):
        return cls.__name__.lower()



# Release process setup see:
# https://github.com/pypa/twine
#
# Upgrade twine
#     python3 -m pip install --user --upgrade twine
#
# Run this to build the `dist/PACKAGE_NAME-xxx.tar.gz` file
#     rm -rf ./dist && python3 setup.py sdist
#
# Check dist/*
#     python3 -m twine check dist/*
#
# Run this to build & upload it to `pypi`, type your account name when prompted.
#     python3 -m twine upload dist/*
#
# In one command line:
#     rm -rf ./dist && python3 setup.py sdist bdist_wheel && python3 -m twine check dist/*
#     rm -rf ./dist && python3 setup.py sdist bdist_wheel && python3 -m twine upload dist/*
#

from setuptools import setup, find_packages

# Usage: python setup.py sdist bdist_wheel

links = []  # for repo urls (dependency_links)

DESCRIPTION = "A python based ORM (Object relational mapping) to make flexible queries and saving new items in the database."
VERSION = "1.0.6"


with open("README.md", "r") as fh:
    LONG_DESCRIPTION = fh.read()

setup(
    name="sql-orm",
    version=VERSION,
    author="Shubham Dipt",
    author_email="shubham.dipt@gmail.com",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    url="https://github.com/shubhamdipt/sql-orm",
    license=open('LICENSE').read(),
    packages=['sql_orm', 'sql_orm.postgresql', 'sql_orm.postgresql.sql'],
    platforms=["any"],
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    dependency_links=links,
)